/**
 * @file
 * VuoList_VuoAudioOutputDevice C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoAudioOutputDevice_H
#define VuoList_VuoAudioOutputDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoAudioOutputDevice VuoList_VuoAudioOutputDevice
 * A list of @ref VuoAudioOutputDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoAudioOutputDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoAudioOutputDevice elements.
 */
typedef const struct VuoList_VuoAudioOutputDevice_struct { void *l; } * VuoList_VuoAudioOutputDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoAudioOutputDevice elements.
 */
VuoList_VuoAudioOutputDevice VuoListCreate_VuoAudioOutputDevice(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoAudioOutputDevice VuoListCopy_VuoAudioOutputDevice(const VuoList_VuoAudioOutputDevice list);

/**
 * Returns the @ref VuoAudioOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoAudioOutputDevice VuoListGetValue_VuoAudioOutputDevice(const VuoList_VuoAudioOutputDevice list, const unsigned long index);

/**
 * Changes the @ref VuoAudioOutputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoAudioOutputDevice(const VuoList_VuoAudioOutputDevice list, const VuoAudioOutputDevice value, const unsigned long index);

/**
 * Inserts the `VuoAudioOutputDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoAudioOutputDevice(const VuoList_VuoAudioOutputDevice list, const VuoAudioOutputDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list, const VuoAudioOutputDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list, const VuoAudioOutputDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list);

/**
 * Removes the `VuoAudioOutputDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoAudioOutputDevice(VuoList_VuoAudioOutputDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoAudioOutputDevice(const VuoList_VuoAudioOutputDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoAudioOutputDevice VuoList_VuoAudioOutputDevice_valueFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoAudioOutputDevice_jsonFromValue(const VuoList_VuoAudioOutputDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoAudioOutputDevice_summaryFromValue(const VuoList_VuoAudioOutputDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoAudioOutputDevice VuoList_VuoAudioOutputDevice_valueFromString(const char *str);
char * VuoList_VuoAudioOutputDevice_stringFromValue(const VuoList_VuoAudioOutputDevice value);
/// @}

/**
 * @}
 */

#endif
